@@ -4,8 +4,7 @@
 #include <utils.h>
 #include <string.h>
 
-int main(int argc, char** argv)
-{
+int main(int argc, char** argv) {
   if (argc != 2) {
     return -1;
   }
